<?php

namespace App\Controller;

use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Attribute\Route;
use KnpU\OAuth2ClientBundle\Client\ClientRegistry;
use Symfony\Component\HttpFoundation\JsonResponse;
use Psr\Log\LoggerInterface;
use Symfony\Component\Security\Core\Exception\AccessDeniedException;

final class DashboardController extends AbstractController
{
    #[Route('/', name: 'app_home')]
    public function home(): Response
    {
        return $this->render('dashboard/home.html.twig', [
            'controller_name' => 'DashboardController',
        ]);
    }
    
    #[Route('/dashboard', name: 'app_dashboard')]
    public function index(): Response
    {
        $this->denyAccessUnlessGranted('ROLE_USER');
        
        return $this->render('dashboard/index.html.twig', [
            'controller_name' => 'DashboardController',
        ]);
    }

    #[Route('/user-info', name: 'app_user_info')]
    public function userInfo(LoggerInterface $logger): Response
    {
        if (!$this->getUser()) {
            throw new AccessDeniedException('You must be logged in to view this page');
        }
        
        $user = $this->getUser();
        $logger->info('User info accessed', [
            'user' => $user->getUserIdentifier()
        ]);
        
        return $this->render('dashboard/user_info.html.twig', [
            'user' => $user
        ]);
    }

    #[Route('/connect/sso', name: 'connect_sso_start')]
    public function connectAction(ClientRegistry $clientRegistry, LoggerInterface $logger): Response
    {
        try {
            $client = $clientRegistry->getClient('sudalys');
            
            $logger->info('Redirecting to OAuth authorization endpoint');
            
            // Don't pass any options - use defaults configured in knpu_oauth2_client.yaml
            return $client->redirect();
        } catch (\Exception $e) {
            $logger->error('OAuth connection error: ' . $e->getMessage(), [
                'exception' => get_class($e),
                'file' => $e->getFile(),
                'line' => $e->getLine()
            ]);
            
            return new JsonResponse([
                'error' => 'OAuth connection failed',
                'message' => $e->getMessage(),
                'trace' => $e->getTraceAsString(),
                'environment' => [
                    'SSO_CLIENT_ID' => $_ENV['SSO_CLIENT_ID'] ?? 'not set',
                    'SSO_CLIENT_SECRET' => substr($_ENV['SSO_CLIENT_SECRET'] ?? 'not set', 0, 3) . '***',
                    'SSO_AUTHORIZE_URL' => $_ENV['SSO_AUTHORIZE_URL'] ?? 'not set',
                    'SSO_TOKEN_URL' => $_ENV['SSO_TOKEN_URL'] ?? 'not set',
                    'SSO_USERINFO_URL' => $_ENV['SSO_USERINFO_URL'] ?? 'not set',
                ]
            ], 500);
        }
    }

    #[Route('/connect/sso/check', name: 'connect_sso_check')]
    public function connectCheckAction(LoggerInterface $logger): Response
    {
        $logger->info('OAuth callback called');
        // This method will be intercepted by the OAuth authenticator
        return new JsonResponse(['status' => 'This should not be displayed if OAuth is working properly']);
    }
    
    #[Route('/logout', name: 'app_logout')]
    public function logout(): void
    {
        // This method can be empty - it will be intercepted by the logout key on your firewall
        throw new \Exception('This should never be reached!');
    }
    
    #[Route('/oauth-debug', name: 'oauth_debug')]
    public function oauthDebug(): Response
    {
        return new JsonResponse([
            'SSO_CLIENT_ID' => $_ENV['SSO_CLIENT_ID'] ?? 'not set',
            'SSO_CLIENT_SECRET' => substr($_ENV['SSO_CLIENT_SECRET'] ?? 'not set', 0, 3) . '***',
            'SSO_AUTHORIZE_URL' => $_ENV['SSO_AUTHORIZE_URL'] ?? 'not set',
            'SSO_TOKEN_URL' => $_ENV['SSO_TOKEN_URL'] ?? 'not set',
            'SSO_USERINFO_URL' => $_ENV['SSO_USERINFO_URL'] ?? 'not set',
        ]);
    }
    
    #[Route('/login-status', name: 'app_login_status')]
    public function loginStatus(): JsonResponse
    {
        $user = $this->getUser();
        
        return new JsonResponse([
            'logged_in' => $user !== null,
            'user' => $user ? [
                'identifier' => $user->getUserIdentifier(),
                'roles' => $user->getRoles(),
            ] : null,
        ]);
    }
}
